# -*- coding: utf-8 -*-

# Standard Library
import json

# Vendor Library

# Project Library
from gisdo.index.constants import (
    APPLICATION_INDEXES,
    QUEUE_INDEXES,
    ENROLLED_INDEXES,
    CAPACITIES_INDEXES)
from gisdo.constants import (
    AGE_CATEGORIES_EIGHT,
    AGE_CATEGORIES_FULL,
    AGE_CATEGORIES_CUT,
    AGE_CATEGORIES_ALL)


class UIProxy(object):

    APPLICATION = 'applications'
    QUEUE = 'queues'
    ENROLLED = 'enrolled'
    CAPACITIES = 'capacities'

    PARTS = {
        APPLICATION: APPLICATION_INDEXES,
        QUEUE: QUEUE_INDEXES,
        ENROLLED: ENROLLED_INDEXES,
        CAPACITIES: CAPACITIES_INDEXES
    }

    def __init__(self, report_form_row):

        self._report_form_row = report_form_row

    def _get_part(self, report_part):

        part = json.loads(
            getattr(self._report_form_row, report_part))

        return part

    @staticmethod
    def _get_index_age_category(part, index_id):

        index = part[index_id]

        for age_class in (AGE_CATEGORIES_CUT,
                          AGE_CATEGORIES_EIGHT,
                          AGE_CATEGORIES_FULL,
                          AGE_CATEGORIES_ALL):
            for age_cat in index:
                if age_cat not in age_class.values():
                    break
            else:
                return age_class

    def _get_ui_part(self, part_name):

        part_ui_data = []
        current_pos = 1
        part = self._get_part(part_name)
        for index_id in self.PARTS[part_name]:

            age_class = UIProxy._get_index_age_category(part, index_id)

            for age_category in age_class:

                count = part[index_id][age_class[age_category]]
                verbose_present = self.PARTS[part_name][index_id]
                try:
                    verbose_present = verbose_present % age_class[age_category]
                except Exception:
                    pass
                part_ui_data.append((current_pos, verbose_present, count))
                current_pos += 1

        return part_ui_data

    def get_ui_data(self):

        applications = self._get_ui_part(self.APPLICATION)
        queues = self._get_ui_part(self.QUEUE)
        enrolled = self._get_ui_part(self.ENROLLED)
        capacities = self._get_ui_part(self.CAPACITIES)

        return (
            applications,
            queues,
            enrolled,
            capacities)
